#ifndef EVENT_LOOP_CXX_H
#define EVENT_LOOP_CXX_H

/**
 * C++ wrapper for <code>event_loop</code>.
 *
 * @file
 * @author Paul Serice
 * @brief C++ wrapper for <code>event_loop</code>
 */

#include "event_loop.h"
#include "cgul_exception_cxx.h"

/**
 * This class provides the C++ bindings for C <code>event_loop</code>
 * objects.  The main purpose of this class is to convert the C-style
 * function calls and exception handling in <code>event_loop</code>
 * into C++-style function calls and exception handling.
 *
 * @brief C++ bindings for <code>event_loop</code>
 */
class event_loop_cxx {

public:

    /**
     * Timeout handle.
     */
    typedef event_loop__timeout_t timeout_t;

    /**
     * File descriptor ready callback.
     */
    typedef event_loop__fd_ready_cb_t fd_ready_cb_t;

    /**
     * Timeout expired callback.
     */
    typedef event_loop__timeout_cb_t timeout_cb_t;

    /**
     * Posix asynchronous signal caught callback.
     */
    typedef event_loop__signal_cb_t signal_cb_t;

    /**
     * Get the <code>event_loop_cxx</code> wrapper for the single
     * instance of the <code>event_loop</code> class.  The single
     * instance is created the first time this method is called.  If
     * the instance cannot be created, an exception is thrown. <p>
     *
     * @note For the main thread, use the singleton returned by this
     * method which can handle Posix asynchronous signals.  Do not use
     * a per-thread instance returned by <code>event_loop_cxx()</code>
     * which cannot handle Posix asynchronous signals. <p>
     *
     * @return single instance wrapper (i.e., main event loop)
     */
    static event_loop_cxx*
    get_instance()
    {
        /* Get the current underlying event_loop singleton.  (It can
         * change if "put_instance()" is called.) */
        cgul_exception_t local = NULL;
        event_loop_t instance = event_loop__get_instance(&local);
        if (local) {
            throw cgul_exception_cxx(local);
        }
        /* The original m_obj is freed early via the static constructor. */
        s_instance_wrapper.m_obj = instance;
        return &s_instance_wrapper;
    }

    /**
     * Put the single instance of the <code>event_loop</code> class.
     * The single instance is destroyed whenever this method is called
     * which invalidates any outstanding <code>event_loop_cxx</code>
     * wrappers for the single instance.  In general, it should only
     * be called before exiting the application if it is called at
     * all.
     */
    static void
    put_instance()
    {
        event_loop__put_instance();
        s_instance_wrapper.m_obj = NULL;
    }

    /**
     * Create a new, per-thread <code>event_loop_cxx</code> object
     * that handles file descriptor and timeout events but not Posix
     * asynchronous signal events.  The caller is responsible for
     * freeing the object by calling the <code>event_loop_cxx</code>
     * destructor.  If memory cannot be allocated, an exception is
     * thrown. <p>
     *
     * @note For the main thread, do not create a new
     * <code>event_loop_cxx</code> object using this method; instead,
     * use the singleton returned by <code>get_instance()</code> which
     * has global visibility and can handle Posix asynchronous
     * signals. <p>
     */
    event_loop_cxx()
    {
        cgul_exception_t local = NULL;
        m_obj = event_loop__new(&local);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * Destructor
     */
    virtual
    ~event_loop_cxx()
    {
        /* Note that set_obj() is programmed to reject any attempt to
         * set "m_obj" to be the "event_loop" singleton so "m_obj"
         * should only ever be a per-thread "event_loop" instance
         * which we are free to delete. */
        if (m_obj) {
            event_loop__delete(m_obj);
            m_obj = NULL;
        }
    }

    /**
     * This method starts the main event loop which only returns when
     * one of the callbacks registered with the event loop invokes
     * <code>stop()</code>.
     */
    virtual void
    start()
    {
        cgul_exception_t local = NULL;
        event_loop__start(&local, m_obj);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * Call this method from within an event callback or from a
     * different thread in order to stop processing events.  This will
     * cause <code>start()</code> to return to its caller.  This
     * method is thread-safe.  If an error occurs, an exception is
     * thrown.
     */
    virtual void
    stop()
    {
        cgul_exception_t local = NULL;
        event_loop__stop(&local, m_obj);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * This method arranges for <code>cb</code> to be called synchronously
     * with <code>cb_data</code> when a Posix asynchronous signal is
     * caught.  The big advantage of this is that <code>cb</code> runs in
     * the synchronous context of the event loop's thread instead of in
     * the asynchronous context of a true signal handler.  If
     * <code>cb</code> is <code>NULL</code>, the signal is effectively
     * ignored.  If the signal cannot be registered, an exception is
     * thrown. <p>
     *
     * You can change <code>cb</code> and <code>cb_data</code> for a
     * particular signal by calling this method more than once, but only
     * one signal handler can simultaneously be registered per signal. <p>
     *
     * As a convenience, the <code>event_loop_cxx</code> class is re-entrant
     * with respect to the event loop's single thread of execution.  This
     * means you can call any of the methods of this class even from
     * callbacks registered with this class. <p>
     *
     * If an error occurs, an exception is thrown.
     *
     * @param[in] signal_number signal number
     * @param[in] cb callback
     * @param[in] cb_data callback data passed to <code>cb</code>
     */
    virtual void
    add_signal_handler(int signal_number,
                       signal_cb_t cb,
                       void* cb_data)
    {
        cgul_exception_t local = NULL;
        event_loop__add_signal_handler(
            &local, m_obj, signal_number, cb, cb_data);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * This method arranges for the C++ event listener
     * <code>listener</code> to be called synchronously on the object
     * <code>obj</code> when a Posix asynchronous signal is caught.  The
     * big advantage of this is that <code>listener</code> runs in the
     * synchronous context of the event loop's thread instead of in the
     * asynchronous context of a true signal handler.  If the signal
     * cannot be registered, an exception is thrown. <p>
     *
     * You can change <code>listener</code> and <code>obj</code> for a
     * particular signal by calling this method more than once, but only
     * one signal handler can simultaneously be registered per signal. <p>
     *
     * When the Posix asynchronous signal is synchronously detected, the
     * C++ "pointer to member function" <code>listener</code> is invoked
     * on the object <code>obj</code> which must be an instance of a class
     * derived from <code>event_loop_listener</code>.  Use the
     * <code>EVENT_LOOP__UPCAST_SIGNAL_HANDLER()</code> macro to properly
     * upcast your "pointer to member function" for your derived class to
     * a type of "pointer to member function" expected by this function.
     * This should look something like the following: <p>
     *
     *   <code>
     *     loop->add_signal_handler(
     *        signal_number, obj,
     *        EVENT_LOOP__UPCAST_SIGNAL_HANDLER(&my_class::my_signal_handler));
     *   </code>
     *
     * As a convenience, the <code>event_loop_cxx</code> class is re-entrant
     * with respect to the event loop's single thread of execution.  This
     * means you can call any of the methods of this class even from
     * callbacks registered with this class. <p>
     *
     * If an error occurs, an exception is thrown.
     *
     * @param[in] signal_number signal number
     * @param[in] obj object that implements the event listener
     * @param[in] listener event listener
     * @see event_loop_listener
     * @see EVENT_LOOP__UPCAST_SIGNAL_HANDLER
     */
    virtual void
    add_signal_handler(
        int signal_number,
        event_loop_listener* obj,
        void (event_loop_listener::*listener)(int))
    {
        cgul_exception_t local = NULL;
        event_loop__add_signal_handler_listener(
            &local, m_obj, signal_number, obj, listener);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * Remove the the signal handler for the signal
     * <code>signal_number</code> from the list of Posix asynchronous
     * signals handled by the event loop.  This causes the default
     * system action for the signal to be restored.  If an error
     * occurs, an exception is thrown.
     *
     * @param[in] signal_number signal number
     */
    virtual void
    remove_signal_handler(int signal_number)
    {
        cgul_exception_t local = NULL;
        event_loop__remove_signal_handler(&local, m_obj, signal_number);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * This method adds a timeout event that repeatedly schedules itself
     * every <code>period</code> seconds.  Because <code>period</code> is
     * a double, timeouts with millisecond resolution can easily be
     * scheduled.  The only guarantee regarding when the timeout occurs is
     * that it occurs at least <code>period</code> seconds after it has
     * been scheduled.  If <code>period</code> is less than zero, the
     * timeout will not be registered. <p>
     *
     * The <code>event_loop_cxx</code> class is re-entrant with
     * respect to the event loop's single thread of execution making
     * it possible to remove a timeout from <code>cb</code>
     * itself. <p>
     *
     * The value returned is the handle for the timeout.  It uniquely
     * identifies the timeout and must be passed into
     * <code>remove_timeout()</code> in order to remove the
     * timeout.  If you plan on removing the callback from inside
     * <code>cb</code>, you do not need to keep a reference to the value
     * that is returned as it is always passed into your callback. <p>
     *
     * If an error occurs, an exception is thrown.
     *
     * @param[in] period timeout period
     * @param[in] cb_data callback data passed to <code>cb</code>
     * @param[in] cb callback
     * @return timeout handle
     */
    virtual timeout_t
    add_timeout(double period,
                timeout_cb_t cb,
                void* cb_data)
    {
        timeout_t rv = NULL;
        cgul_exception_t local = NULL;
        rv = event_loop__add_timeout(&local, m_obj, period, cb, cb_data);
        if (local) {
            throw cgul_exception_cxx(local);
        }
        return rv;
    }

    /**
     * This method adds a C++ timeout event listener that repeatedly
     * schedules itself every <code>period</code> seconds.  Because
     * <code>period</code> is a double, timeouts with millisecond
     * resolution can easily be scheduled.  The only guarantee regarding
     * when the timeout occurs is that it occurs at least
     * <code>period</code> seconds after it has been scheduled.  If
     * <code>period</code> is less than zero, the timeout will not be
     * registered. <p>
     *
     * When the timeout expires, the C++ "pointer to member function"
     * <code>listener</code> is invoked on the object <code>obj</code>
     * which must be an instance of a class derived from
     * <code>event_loop_listener</code>.  Use the
     * <code>EVENT_LOOP__UPCAST_TIMEOUT()</code> macro to properly upcast
     * your "pointer to member function" for your derived class to a type
     * of "pointer to member function" expected by this function.  This
     * should look something like the following: <p>
     *
     *   <code>
     *       loop->add_timeout_listener(
     *           period, obj,
     *           EVENT_LOOP__UPCAST_TIMEOUT(&my_class::my_timeout));
     *   </code>
     *
     * The <code>event_loop_cxx</code> class is re-entrant with
     * respect to the event loop's single thread of execution making
     * it possible to remove a timeout from <code>listener</code>
     * itself. <p>
     *
     * The value returned is the handle for the timeout.  It uniquely
     * identifies the timeout and must be passed into
     * <code>remove_timeout()</code> in order to remove the timeout.
     * If you plan on removing the listener from inside
     * <code>listener</code>, you do not need to keep a reference to
     * the value that is returned as it is always passed into your
     * listener. <p>
     *
     * If an error occurs, an exception is thrown.
     *
     * @param[in] period timeout period
     * @param[in] obj object that implements the event listener
     * @param[in] listener event listener
     * @return timeout handle
     * @see event_loop_listener
     * @see EVENT_LOOP__UPCAST_TIMEOUT
     */
    virtual timeout_t
    add_timeout(
        double period,
        event_loop_listener* obj,
        void (event_loop_listener::*listener)(timeout_t))
    {
        timeout_t rv = NULL;
        cgul_exception_t local = NULL;
        rv = event_loop__add_timeout_listener(
                 &local, m_obj, period, obj, listener);
        if (local) {
            throw cgul_exception_cxx(local);
        }
        return rv;
    }

    /**
     * This function removes timeouts registered with
     * <code>add_timeout()</code>.  After calling this function,
     * <code>timeout</code> is invalid.  If an error occurs, an
     * exception is thrown. <p>
     *
     * This method can throw an out-of-memory exception.  This can happen
     * when this method is called from any timeout callback because this
     * causes this class to be re-entered which requires that the request
     * to remove the timeout be added to a queue so it can be honored
     * later when it is safe to do so. <p>
     *
     * @param[in] timeout timeout handle
     */
    virtual void
    remove_timeout(timeout_t timeout)
    {
        cgul_exception_t local = NULL;
        event_loop__remove_timeout(&local, m_obj, timeout);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * This method adds <code>fd</code> to the list of file descriptors
     * the event loop monitors.  The event loop will then call
     * <code>cb</code> with the client-supplied <code>cb_data</code>
     * pointer when <code>fd</code> is ready to be read. <p>
     *
     * You can change <code>cb</code> and <code>cb_data</code> for a
     * particular file descriptor by calling this method more than once,
     * but only one "read" callback per file descriptor can simultaneously
     * be registered. <p>
     *
     * As a convenience, the <code>event_loop_cxx</code> class is re-entrant
     * with respect to the event loop's single thread of execution.  This
     * means you can call any of the methods of this class even from
     * callbacks registered with this class. <p>
     *
     * If an error occurs, an exception is thrown. <p>
     *
     * @param[in] fd file descriptor to monitor
     * @param[in] cb callback
     * @param[in] cb_data callback data passed to <code>cb</code>
     */
    virtual void
    add_read_fd(int fd,
                fd_ready_cb_t cb,
                void* cb_data)
    {
        cgul_exception_t local = NULL;
        event_loop__add_read_fd(&local, m_obj, fd, cb, cb_data);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * This method adds <code>fd</code> to the list of file descriptors
     * the event loop monitors.  You can change <code>listener</code> and
     * <code>obj</code> for a particular file descriptor by calling this
     * method more than once, but only one "read" event listener per file
     * descriptor can simultaneously be registered. <p>
     *
     * When the file descriptor is ready, the C++ "pointer to member
     * function" <code>listener</code> is invoked on the object
     * <code>obj</code> which must be an instance of a class derived from
     * <code>event_loop_listener</code>.  Use the
     * <code>EVENT_LOOP__UPCAST_FD_READY()</code> macro to properly upcast
     * your "pointer to member function" for your derived class to a type
     * of "pointer to member function" expected by this function.  This
     * should look something like the following: <p>
     *
     *   <code>
     *       loop->add_read_fd(
     *           fd, obj,
     *           EVENT_LOOP__UPCAST_FD_READY(&my_class::my_listener));
     *   </code>
     *
     * As a convenience, the <code>event_loop_cxx</code> class is re-entrant
     * with respect to the event loop's single thread of execution.  This
     * means you can call any of the methods of this class even from event
     * listeners registered with this class. <p>
     *
     * If an error occurs, an exception is thrown. <p>
     *
     * @param[in] fd file descriptor to monitor
     * @param[in] obj object that implements the event listener
     * @param[in] listener event listener
     * @see event_loop_listener
     * @see EVENT_LOOP__UPCAST_FD_READY
     */
    virtual void
    add_read_fd(
        int fd,
        event_loop_listener* obj,
        void (event_loop_listener::*listener)(int))
    {
        cgul_exception_t local = NULL;
        event_loop__add_read_fd_listener(&local, m_obj, fd, obj, listener);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * Remove <code>fd</code> from the list of file descriptors the
     * event loop monitors to see when they are ready to be read.  If
     * an error occurs, an exception is thrown.
     *
     * @param[in] fd file descriptor to remove
     */
    virtual void
    remove_read_fd(int fd)
    {
        cgul_exception_t local = NULL;
        event_loop__remove_read_fd(&local, m_obj, fd);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * This method adds <code>fd</code> to the list of file descriptors
     * the event loop monitors.  The event loop will then call
     * <code>cb</code> with the client-supplied <code>cb_data</code>
     * pointer when <code>fd</code> is ready to be written. <p>
     *
     * You can change <code>cb</code> and <code>cb_data</code> for a
     * particular file descriptor by calling this method more than once,
     * but only one "write" callback per file descriptor can
     * simultaneously be registered. <p>
     *
     * As a convenience, the <code>event_loop_cxx</code> class is re-entrant
     * with respect to the event loop's single thread of execution.  This
     * means you can call any of the methods of this class even from
     * callbacks registered with this class. <p>
     *
     * If an error occurs, an exception is thrown. <p>
     *
     * @param[in] fd file descriptor to monitor
     * @param[in] cb callback
     * @param[in] cb_data callback data passed to <code>cb</code>
     */
    virtual void
    add_write_fd(int fd,
                 fd_ready_cb_t cb,
                 void* cb_data)
    {
        cgul_exception_t local = NULL;
        event_loop__add_write_fd(&local, m_obj, fd, cb, cb_data);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * This method adds <code>fd</code> to the list of file descriptors
     * the event loop monitors.  You can change <code>listener</code> and
     * <code>obj</code> for a particular file descriptor by calling this
     * method more than once, but only one "write" event listener per file
     * descriptor can simultaneously be registered. <p>
     *
     * When the file descriptor is ready, the C++ "pointer to member
     * function" <code>listener</code> is invoked on the object
     * <code>obj</code> which must be an instance of a class derived from
     * <code>event_loop_listener</code>.  Use the
     * <code>EVENT_LOOP__UPCAST_FD_READY()</code> macro to properly upcast
     * your "pointer to member function" for your derived class to a type
     * of "pointer to member function" expected by this function.  This
     * should look something like the following: <p>
     *
     *   <code>
     *       loop->add_write_fd(
     *           fd, obj,
     *           EVENT_LOOP__UPCAST_FD_READY(&my_class::my_listener));
     *   </code>
     *
     * As a convenience, the <code>event_loop_cxx</code> class is re-entrant
     * with respect to the event loop's single thread of execution.  This
     * means you can call any of the methods of this class even from event
     * listeners registered with this class. <p>
     *
     * If an error occurs, an exception is thrown. <p>
     *
     * @param[in] fd file descriptor to monitor
     * @param[in] obj object that implements the event listener
     * @param[in] listener event listener
     * @see event_loop_listener
     * @see EVENT_LOOP__UPCAST_FD_READY
     */
    virtual void
    add_write_fd(
        int fd,
        event_loop_listener* obj,
        void (event_loop_listener::*listener)(int))
    {
        cgul_exception_t local = NULL;
        event_loop__add_write_fd_listener(&local, m_obj, fd, obj, listener);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * Remove <code>fd</code> from the list of file descriptors the
     * event loop monitors to see when they are ready to be written.
     * If an error occurs, an exception is thrown.
     *
     * @param[in] fd file descriptor to remove
     */
    virtual void
    remove_write_fd(int fd)
    {
        cgul_exception_t local = NULL;
        event_loop__remove_write_fd(&local, m_obj, fd);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * This method adds <code>fd</code> to the list of file descriptors
     * the event loop monitors.  The event loop will then call
     * <code>cb</code> with the client-supplied <code>cb_data</code>
     * pointer when <code>fd</code> has an exception. <p>
     *
     * You can change <code>cb</code> and <code>cb_data</code> for a
     * particular file descriptor by calling this method more than once,
     * but only one "exception" callback per file descriptor can
     * simultaneously be registered. <p>
     *
     * As a convenience, the <code>event_loop_cxx</code> class is re-entrant
     * with respect to the event loop's single thread of execution.  This
     * means you can call any of the methods of this class even from
     * callbacks registered with this class. <p>
     *
     * If an error occurs, an exception is thrown. <p>
     *
     * @param[in] fd file descriptor to monitor
     * @param[in] cb callback
     * @param[in] cb_data callback data passed to <code>cb</code>
     */
    virtual void
    add_exception_fd(int fd,
                     fd_ready_cb_t cb,
                     void* cb_data)
    {
        cgul_exception_t local = NULL;
        event_loop__add_exception_fd(&local, m_obj, fd, cb, cb_data);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * This method adds <code>fd</code> to the list of file descriptors
     * the event loop monitors.  You can change <code>listener</code> and
     * <code>obj</code> for a particular file descriptor by calling this
     * method more than once, but only one "exception" event listener per
     * file descriptor can simultaneously be registered. <p>
     *
     * When the file descriptor is ready, the C++ "pointer to member
     * function" <code>listener</code> is invoked on the object
     * <code>obj</code> which must be an instance of a class derived from
     * <code>event_loop_listener</code>.  Use the
     * <code>EVENT_LOOP__UPCAST_FD_READY()</code> macro to properly upcast
     * your "pointer to member function" for your derived class to a type
     * of "pointer to member function" expected by this function.  This
     * should look something like the following: <p>
     *
     *   <code>
     *       loop->add_exception_fd(
     *           fd, obj,
     *           EVENT_LOOP__UPCAST_FD_READY(&my_class::my_listener));
     *   </code>
     *
     * As a convenience, the <code>event_loop_cxx</code> class is re-entrant
     * with respect to the event loop's single thread of execution.  This
     * means you can call any of the methods of this class even from event
     * listeners registered with this class. <p>
     *
     * If an error occurs, an exception is thrown. <p>
     *
     * @param[in] fd file descriptor to monitor
     * @param[in] obj object that implements the event listener
     * @param[in] listener event listener
     * @see event_loop_listener
     * @see EVENT_LOOP__UPCAST_FD_READY
     */
    virtual void
    add_exception_fd(
        int fd,
        event_loop_listener* obj,
        void (event_loop_listener::*listener)(int))
    {
        cgul_exception_t local = NULL;
        event_loop__add_exception_fd_listener(&local, m_obj, fd, obj, listener);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * Remove <code>fd</code> from the list of file descriptors the
     * event loop monitors to see when an exception occurs.  If an
     * error occurs, an exception is thrown.
     *
     * @param[in] fd file descriptor to remove
     */
    virtual void
    remove_exception_fd(int fd)
    {
        cgul_exception_t local = NULL;
        event_loop__remove_exception_fd(&local, m_obj, fd);
        if (local) {
            throw cgul_exception_cxx(local);
        }
    }

    /**
     * Get the underlying <code>event_loop</code> object.
     *
     * @return underlying object
     */
    virtual event_loop_t
    get_obj() const
    {
        return m_obj;
    }

    /**
     * Take the underlying <code>event_loop</code> object.  This means
     * the underlying object will not be deleted when the wrapper goes
     * out of scope.  Also, because you have taken the underlying
     * object, no other methods should be called on this wrapper's
     * instance.  Lastly, after taking the underlying object, it is
     * the caller's responsibility to delete the underlying object by
     * calling <code>event_loop__delete()</code>.
     *
     * @return underlying object
     */
    virtual event_loop_t
    take_obj()
    {
        event_loop_t rv = m_obj;
        m_obj = NULL;
        return rv;
    }

    /**
     * Set the new underlying object to <code>rhs</code>.  This causes
     * the old underlying object to be deleted which invalidates any
     * outstanding pointers to or iterators for the old underlying
     * object.  Only, per-thread <code>event_loop</code> instances can
     * be set; if <code>rhs</code> is the <code>event_loop</code>
     * singleton, an exception is thrown. <p>
     *
     * This instance takes ownership of <code>rhs</code> which means
     * <code>rhs</code> will be automatically deleted when the C++
     * wrapper is deleted.  To prevent automatic deletion of
     * <code>rhs</code>, call <code>take_obj()</code> when the C++
     * wrapper is no longer needed. <p>
     *
     * @param[in] rhs right-hand side
     */
    virtual void
    set_obj(event_loop_t rhs)
    {
        /* Do not let the user set the underlying object to be the
         * singleton "event_loop" instance because access to the
         * singleton should only be through the get_instance() method;
         * otherwise, the singleton would be deleted when this object
         * goes out of scope. */
        if (rhs && (rhs != m_obj) && (rhs != get_instance()->get_obj())) {
            if (m_obj) {
                event_loop__delete(m_obj);
            }
            m_obj = rhs;
        }
    }

private:

    /* Hide these. */
    event_loop_cxx(const event_loop_cxx&);
    event_loop_cxx& operator=(const event_loop_cxx&);

    /* Static Constructor */
    class StaticConstructor {
    public:
        StaticConstructor();
    };

    /** Static Constructor */
    static StaticConstructor s_staticConstructor;

    /** Singleton wrapper for C event_loop singleton. */
    static event_loop_cxx s_instance_wrapper;

    /** Underlying <code>event_loop</code> object. */
    event_loop_t m_obj;

};

#endif  /*  EVENT_LOOP_CXX_H  */
