#ifndef EVENT_LOOP_P_H
#define EVENT_LOOP_P_H

#include <sys/select.h>
#include <sys/time.h>
#include "cgul.h"
#include "event_loop.h"
#include "event_loop_common.h"

CGUL_BEGIN_C

/**
 * Data for timeout events.
 */
typedef struct {

    /** Period for the timeout. */
    double period;

    /** When is this timeout scheduled to expire next. */
    double timeout;

    /** Callback invoked when timeout expires. */
    event_loop__timeout_cb_t cb;

    /** Data to pass <code>cb</code>. */
    void* cb_data;

#ifdef __cplusplus

    /** Object that implements the event listener. */
    event_loop_listener* obj;

    /** Event listener. */
    void (event_loop_listener::*listener)(event_loop__timeout_t);

#endif  /*  __cplusplus */

} *event_loop__timeout_data_t;


/**
 * Data for signal events.
 */
typedef struct event_loop__signal_data {

    /** The signal for which <code>cb</code> is registered. */
    int signal_number;

    /** Callback invoked when the signal is received. */
    event_loop__signal_cb_t cb;

    /** Data to pass <code>cb</code>. */
    void* cb_data;

#ifdef __cplusplus

    /** Object that implements the event listener. */
    event_loop_listener* obj;

    /** Event listener. */
    void (event_loop_listener::*listener)(int);

#endif  /*  __cplusplus */

} *event_loop__signal_data_t;


/**
 * Data for file descriptor events.
 */
typedef struct {

    /** The file descriptor for which <code>cb</code> is registered. */
    int fd;

    /** Callback invoked when <code>fd</code> is ready. */
    event_loop__fd_ready_cb_t cb;

    /** Data to pass <code>cb</code>. */
    void* cb_data;

#ifdef __cplusplus

    /** Object that implements the event listener. */
    event_loop_listener* obj;

    /** Event listener. */
    void (event_loop_listener::*listener)(int);

#endif  /*  __cplusplus */

    /**
     * Because the <code>rset_tree</code>, <code>wset_tree</code>, and
     * <code>eset_tree</code> have so much in common, most of the code
     * that manipulates the fd trees can handle all three trees.  The
     * caller just has to be able to pass in the correct tree. <p>
     *
     * To that end, when a file descriptor ends up on the pending
     * list, we need to be able to direct the pending request to the
     * correct destination tree.
     */
    cgul_rbtree_t destination_tree;

    /**
     * Like <code>destination_tree</code>, this member is used to be
     * able to direct the pending reequest to the correct destination
     * file descriptor set.
     */
    fd_set* destination_set;

} *event_loop__fd_data_t;

/**
 * When exhausting the <code>pending_list</code>, we make callbacks to
 * functions that match this prototype.
 */
typedef void (*event_loop__pending_cb_t)(cgul_exception_t*,
                                         event_loop_t,
                                         void*);

/**
 * Data for making the callbacks that exhaust <code>pending_list</code>.
 */
typedef struct {

    /* Callback function. */
    event_loop__pending_cb_t f;

    /* Client data passed into <code>f</code>. */
    void* data;

} *event_loop__pending_data_t;

/**
 * Data encapsulated by the <code>event_loop</code> class.
 */
struct event_loop {

    /** Whether the event loop should stop and return to the caller. */
    int stop;

    /**
     * Pipe used for thread-safe internal communication.  This is used
     * to wake up <code>select()</code> perhaps from a different
     * thread for things like <code>event_loop__stop()</code>.  To be
     * thread-safe, the method called by the user should only write a
     * message value to the pipe.  This will trigger the event loop
     * thread to wake up and process the message.  It is only when
     * processing the message event that it is safe to modify the
     * <code>event_loop</code>.
     */
    int internal_pipe[2];

    /**
     * Collection of timeout event data sorted by when the timeout is
     * due to expire next.
     */
    cgul_rbtree_t timeout_tree;

    /** Collection of signal event data. */
    cgul_rbtree_t signal_tree;

    /** Collection of readable file descriptor event data. */
    cgul_rbtree_t rset_tree;

    /** Collection of writable file descriptor event data. */
    cgul_rbtree_t wset_tree;

    /** Collection of exception file descriptor event data. */
    cgul_rbtree_t eset_tree;

    /** For re-entrancy purposes, is <code>timeout_tree</code> busy. */
    int timeout_tree_is_busy;

    /** For re-entrancy purposes, is <code>signal_tree</code> busy. */
    int signal_tree_is_busy;

    /** For re-entrancy purposes, is <code>reset_tree</code> busy. */
    int rset_tree_is_busy;

    /** For re-entrancy purposes, is <code>wset_tree</code> busy. */
    int wset_tree_is_busy;

    /** For re-entrancy purposes, is <code>eset_tree</code> busy. */
    int eset_tree_is_busy;

    /**
     * When one of the lists above is busy, we queue "add" or "remove"
     * requests to this list.  This allows this class be be re-entrant
     * because callbacks can add and remove other callbacks.  We only
     * need one pending list because an event loop is, by definition,
     * single threaded.  Thus, the event loop only ever makes one
     * callback at a time.  Thus, at most, only one callback lists can
     * every be busy making callbacks.
     */
    cgul_list_t pending_list;

    /** File descriptor with the largest integer value. */
    int max_fd;

    /** Set of all file descriptors waiting for readable events. */
    fd_set rset;

    /** Set of all file descriptors waiting for writable events. */
    fd_set wset;

    /** Set of all file descriptors waiting for exception events. */
    fd_set eset;

};

CGUL_END_C

#endif  /*  EVENT_LOOP_P_H  */
