#include "event_loop.h"
#include <signal.h>
#include <stdio.h>
#include "cgul.h"

/* NOTE: The event loop can easily be set up to convert asynchronous
 * signal handlers into synchronous signal handlers. */
static void
handle_sigint(cgul_exception_t* cex,
              int signal_number,
              void* data)
{
    event_loop_t loop = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Get the event loop. */
    loop = event_loop__get_instance(cex);
    if (*cex) {
        goto out;
    }

    printf("SIGINT (%d) caught.  Exiting!\n", signal_number);

    /* Stop the event loop. */
    event_loop__stop(cex, loop);

 out:
    return;
}

static void
stdin_readable(cgul_exception_t* cex,
               int fd,
               void* data)
{
    const char* tmp = NULL;
    cgul_string_t s = NULL;

    s = cgul_string__new(cex);
    if (*cex) {
        goto out;
    }

    /* Get the next line of input. */
    cgul_get_line_relaxed(cex, stdin, s);
    tmp = cgul_string__get_value(cex, s);
    if (*cex) {
        goto out;
    }

    if (strcmp(tmp, "exit") == 0) {

        /* Stop the event loop so we can exit. */
        event_loop_t loop = event_loop__get_instance(cex);
        event_loop__stop(cex, loop);
        if (*cex) {
            goto out;
        }

    } else if (strcmp(tmp, "disconnect") == 0) {

        /* Test re-entrant code by removing this callback from the
         * callback itself. */
        event_loop_t loop = event_loop__get_instance(cex);
        event_loop__remove_read_fd(cex, loop, fd);
        if (*cex) {
            goto out;
        }

    } else {

        /* Echo string back to user. */
        puts(tmp);

    }

 out:

    /* Clean up. */
    if (s) {
        cgul_string__delete(s);
    }

}

static void
rapid(cgul_exception_t* cex,
      event_loop__timeout_t timeout,
      void* data)
{
    static int count = 0;

    ++count;
    printf("rapid (%d of 10)\n", count);
    fflush(stdout);

    if (count >= 10) {
        event_loop_t loop = event_loop__get_instance(cex);
        event_loop__remove_timeout(cex, loop, timeout);
        if (*cex) {
            goto out;
        }
    }

 out:
    return;
}

static void
tick(cgul_exception_t* cex,
     event_loop__timeout_t timeout,
     void* data)
{
    static int first_pass = 1;

    puts("tick: type something!");

    /* Add the "rapid" timeout if this is the first pass. */
    if (first_pass) {
        first_pass = 0;
        event_loop_t loop = event_loop__get_instance(cex);
        event_loop__add_timeout(cex, loop, 0.25, &rapid, NULL);
        if (*cex) {
            goto out;
        }
    }

 out:
    return;
}

static void
tock(cgul_exception_t* cex,
     event_loop__timeout_t timeout,
     void* data)
{
    puts("tock: type something!");
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    event_loop_t loop = NULL;
    cgul_exception_t cex = NULL;

    /* Get the event_loop instance. */
    loop = event_loop__get_instance(&cex);

    /* Add events you are interested in receiving .*/
    event_loop__add_signal_handler(
        &cex, loop, SIGINT, &handle_sigint, NULL);
    event_loop__add_read_fd(
        &cex, loop, fileno(stdin), &stdin_readable, NULL);
    event_loop__add_timeout(
        &cex, loop, 2.0, &tick, NULL);
    event_loop__add_timeout(
        &cex, loop, 6.5, &tock, NULL);

    /* Enter the main loop. */
    event_loop__start(&cex, loop);

    if (cex) {
        const char* msg = cgul_exception__get_message(cex);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(cex);
        cex = NULL;
    }

    /* Clean up. */
    event_loop__put_instance();

    return rv;
}
